###################################################################################################
# _metadictmanager.py
#
# Implementation of class _metadictmanager.MetadictManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
import AccessControl.Role
import copy
import string
import urllib
# Product Imports.
import _xmllib


# -------------------------------------------------------------------------------------------------
#  getAttrName
# -------------------------------------------------------------------------------------------------
def getAttrName(id):
  s_attr = id
  s_attr = string.lower(s_attr)
  s_attr = string.replace(s_attr,'.','_')
  s_attr = 'attr_%s'%s_attr
  return s_attr

# -------------------------------------------------------------------------------------------------
#  importMetaDict
# -------------------------------------------------------------------------------------------------

def _importMetaDict(self, item):
  attr = None
  id = item['id']
  name = item['name']
  metatype = item['type']
  keys = item['keys']
  dst_meta_types = item['dst_meta_types']
  self.__set_metadict__(attr,id,name,metatype,keys,dst_meta_types)

def importMetaDict(self, xml):
  builder = _xmllib.XmlAttrBuilder()
  value = builder.parse(xml)
  if type(value) is type([]):
    for item in value:
      _importMetaDict(self,item)
  else:
    _importMetaDict(self,value)
  self.synchronizeObjAttrs()


# -------------------------------------------------------------------------------------------------
#  changeMetaDictAttrs
# -------------------------------------------------------------------------------------------------
def changeMetaDictAttrs(self, old_attr, new_attr):
  if self.meta_type[:3] == 'ZMS':
    for s_preview in [None,'preview']:
      req = {'preview':s_preview}
      ob = self.getObjVersion(req)
      for s_lang in self.getLanguages():
        old_lang_attr = "%s_%s"%(old_attr,s_lang)
        new_lang_attr = "%s_%s"%(new_attr,s_lang)
        if hasattr(ob,old_lang_attr):
          try:
            setattr(ob,new_lang_attr,copy.deepcopy(getattr(ob,old_lang_attr)))
          except:
            pass
    for ob in self.objectValues():
      changeMetaDictAttrs(ob,old_attr,new_attr)

# -------------------------------------------------------------------------------------------------
#  deleteMetaDictAttrs
# -------------------------------------------------------------------------------------------------
def deleteMetaDictAttrs(self, attr):
  if self.meta_type[:3] == 'ZMS':
    for s_lang in self.getLanguages():
      lang_attr = "%s_%s"%(attr,s_lang)
      if hasattr(self,lang_attr):
        try:
          delattr(self,lang_attr)
        except:
          pass
    for ob in self.objectValues():
      deleteMetaDictAttrs(ob,attr)


###################################################################################################
###################################################################################################
###
###   class MetadictManager
###
###################################################################################################
###################################################################################################
class MetadictManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
                'manage_changeMetadicts','manage_customizeMetadictForm',
                )
    __ac_permissions__=(
                ('ZMS Administrator', __administratorPermissions__),
                ('ZMS Author', __authorPermissions__),
                ('ZMS Editor', __editorPermissions__),
                )
                

    # Management Interface to customize DC.Metadictionaries.
    manage_customizeMetadictForm = HTMLFile('dtml/metadict/manage_customizeform', globals()) 

    # ------------------------------------------------------------------------------------------
    # MetadictManager.recurse_updateVersionMetaDicts: 
    #
    # Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersionMetaDicts(self, lang, manage_lang, REQUEST):
      message = ""
      
      # ---------------------------------------------------------------------
      # 26.03.2002
      # Build #069: Remove multiple.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '069':
        old_metadicts = self.__get_metadicts__()
        obs = []
        for old_metadict in old_metadicts:
          if type(old_metadict) is type({}):
            new_metadict = {}
            for key in old_metadict.keys():
              if key not in ['multiple']:
                new_metadict[key] = old_metadict[key]
          else:
            new_metadict = old_metadict
          obs.append(new_metadict)
        self.setConfProperty('ZMS.custom.metas',copy.deepcopy(obs))
      
      ##### SYNCHRONIZE ####
      self.synchronizeObjAttrs()
      
      return message


    # ------------------------------------------------------------------------------------------
    # MetadictManager.__get_metadicts__:
    # 
    # Returns list of DC.Metadictionaries.
    # ------------------------------------------------------------------------------------------
    def __get_metadicts__(self):
      return self.getConfProperty('ZMS.custom.metas',[])


    # ------------------------------------------------------------------------------------------
    # MetadictManager.getMetadictAttrs:
    # 
    # Returns list of attributes of DC.Metadictionaries.
    # ------------------------------------------------------------------------------------------
    def getMetadictAttrs(self, meta_type=None):
      attrs = []
      obs = self.__get_metadicts__()
      for i in range(0,len(obs)/2):
        attr = obs[i*2]
        metadictDetails = self.getMetadictDetails(attr)
        if meta_type is None or meta_type in metadictDetails.getDstMetaTypes():
          attrs.append(attr)
      # ZMSLinkElement must always have attr_dc_description!
      if meta_type == 'ZMSLinkElement':
        if not 'attr_dc_description' in attrs:
          attrs.insert(0,'attr_dc_description')
      # Return attributes.
      return attrs


    # ------------------------------------------------------------------------------------------
    # MetadictManager.getMetadictDetails:
    # 
    # Returns Details of DC.Metadictionary identified by attribute-name.
    # ------------------------------------------------------------------------------------------
    def getMetadictDetails(self, attr):
      obs = self.__get_metadicts__()
      if attr in obs:
        ob = obs[obs.index(attr) + 1]
      # ZMSLinkElement must always have attr_dc_description!
      elif attr == 'attr_dc_description':
        manage_lang = self.getManageLanguage(self.getPrimaryLanguage())
        ob = {}
        ob['id'] = 'DC.Description'
        ob['name'] = self.getLangStr('ATTR_DC_DESCRIPTION',manage_lang)
        ob['type'] = 'textarea'
        ob['keys'] = []
        ob['dst_meta_types'] = [self.meta_type]
      return MetadictDetails(ob)


    # ------------------------------------------------------------------------------------------
    # MetadictManager.__set_metadict__:
    # 
    # Set/add DC.Metadictionary with specified values.
    # ------------------------------------------------------------------------------------------
    def __set_metadict__(self, attr, id, name, metatype, keys='', dst_meta_types=[]):
      # Remove exisiting entry.
      obs = self.__get_metadicts__()
      if attr is not None:
        i = obs.index(attr)
        del obs[i] 
        del obs[i] 
      else: 
        i = len(obs)
      # Attribute.
      attr = getAttrName(id)
      # Values.
      values = {}
      values['id'] = id
      values['name'] = name
      values['type'] = metatype
      # Keys.
      values['keys'] = []
      if type(keys) is type(''):
        for key in string.split(keys,'\n'):
          key = string.strip(key)
          if len(key) > 0:
            values['keys'].append(key)
      elif type(keys) is type([]):
        values['keys'] = keys
      # Meta-Types.
      if not type(dst_meta_types) is type([]):
        dst_meta_types = [dst_meta_types]
      values['dst_meta_types'] = dst_meta_types
      # Update attribute.
      obs.insert(i,values)
      obs.insert(i,attr)
      self.setConfProperty('ZMS.custom.metas',copy.deepcopy(obs))


    # ------------------------------------------------------------------------------------------
    #	MetadictManager.__move_metadict__:
    #
    #	Moves DC.Metadictionary with specified ATTR in specified DIRECTION.
    # ------------------------------------------------------------------------------------------
    def __move_metadict__(self, attr, dir):
      # Move.
      obs = self.__get_metadicts__()
      i = obs.index(attr)
      attr = obs[i] # Attribute
      values = obs[i+1] # Values
      del obs[i] 
      del obs[i] 
      # Update attribute.
      obs.insert(i+dir*2,values)
      obs.insert(i+dir*2,attr)
      self.setConfProperty('ZMS.custom.metas',copy.deepcopy(obs))


    # ------------------------------------------------------------------------------------------
    # MetadictManager.__del_metadict__:
    # 
    # Delete DC.Metadictionary with specified ATTR.
    # ------------------------------------------------------------------------------------------
    def __del_metadict__(self, attr):
      # Delete.
      obs = self.__get_metadicts__()
      i = obs.index(attr)
      # Update attribute.
      del obs[i] # Attribute
      del obs[i] # Values
      self.setConfProperty('ZMS.custom.metas',copy.deepcopy(obs))


    ###############################################################################################
    # MetadictManager.manage_changeMetadicts:
    #
    # Werte fr DC.Metadictionaries lschen oder ndern.
    ###############################################################################################
    def manage_changeMetadicts(self, btn, lang, manage_lang, REQUEST, RESPONSE=None):
        """ MetadictManager.manage_changeMetadicts """        

        message = ''
        attr = REQUEST.get('attr','')
        
        
        # Delete.
        # -------
        if btn in ['delete',self.getLangStr('BTN_DELETE',manage_lang)]:
          self.__del_metadict__(attr)
          deleteMetaDictAttrs(self,attr)
          message = self.getLangStr('MSG_CHANGED',manage_lang)
          attr = ''
                    
        # Change.
        # -------
        elif btn == self.getLangStr('BTN_CHANGE',manage_lang): 
          # Change.
          id = string.strip(REQUEST['id'])
          name = string.strip(REQUEST['name'])
          metatype = string.strip(REQUEST['metatype'])
          keys = string.strip(REQUEST['_keys'])
          dst_meta_types = []
          for key in self.dGlobalAttrs.keys():
            if self.dGlobalAttrs[key]['meta_dict'] == 1:
              if REQUEST.has_key('_%s_dct_meta_type'%key) and \
              	 REQUEST['_%s_dct_meta_type'%key] == 1:
              	dst_meta_types.append(key)
          self.__set_metadict__(attr,id,name,metatype,keys,dst_meta_types)
          #
          new_attr = getAttrName(id)
          if attr != new_attr:
            changeMetaDictAttrs(self,attr,new_attr)
            deleteMetaDictAttrs(self,attr)
          # Display attribute..
          attr = new_attr
          message = self.getLangStr('MSG_CHANGED',manage_lang)
        
        # Export.
        # -------
        elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
          value = []
          attrs = REQUEST.get('attrs',[])
          metadicts = self.__get_metadicts__()
          for i in range(0,len(metadicts)/2):            
            id = metadicts[i*2]
            dict = metadicts[i*2+1]
            if id in attrs or len(attrs) == 0:
              value.append(dict)
          if len(value)==1:
            value = value[0]
          content_type = 'text/xml'
          filename = 'ZMS.custom.metas.xml'
          export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
          export = export + _xmllib.toXml(self,value,1)
          RESPONSE.setHeader('Content-Type',content_type)
          RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
          return export

        # Import.
        # -------
        elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
          importMetaDict(self,xml=REQUEST['file'])
          message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%REQUEST['file'].filename)

        # Move up.
        # --------
        elif btn == 'move_up':
          self.__move_metadict__(attr,-1)
          message = self.getLangStr('MSG_MOVEDOBJUP',manage_lang)%("<i>%s</i>"%attr)
          attr = ''
        
        # Move down.
        # ----------
        elif btn == 'move_down':
          self.__move_metadict__(attr,+1)
          message = self.getLangStr('MSG_MOVEDOBJDOWN',manage_lang)%("<i>%s</i>"%attr)
          attr = ''
        
        # Insert.
        # -------
        elif btn == self.getLangStr('BTN_INSERT',manage_lang): 
          id = string.strip(REQUEST['_id'])
          name = string.strip(REQUEST['_name'])
          metatype = string.strip(REQUEST['_type'])
          self.__set_metadict__(None,id,name,metatype)
          # Display attribute..
          attr = getAttrName(id)
          
        ##### SYNCHRONIZE ####
        self.synchronizeObjAttrs()
        
        # Return with message.
        message = urllib.quote(message)
        return REQUEST.RESPONSE.redirect('manage_customizeMetadictForm?attr=%s&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(attr,lang,manage_lang,message))


# -------------------------------------------------------------------------------------------------
#  CLASS METADICTDETAILS
# -------------------------------------------------------------------------------------------------
class MetadictDetails(AccessControl.Role.RoleManager):

  # Constructor.
  # ------------
  def __init__(self, ob):
    self.setId(ob['id'])
    self.setName(ob['name'])
    self.setKeys(ob['keys'])
    self.setType(ob['type'])
    # Attr
    self.setAttr(getAttrName(self.getId()))
    # dst_meta_types
    if ob.has_key('dst_meta_types'):
      dst_meta_types = ob['dst_meta_types']
    else:
      dst_meta_types = ['ZMS','ZMSRubrik','ZMSDocument','ZMSFile','ZMSGraphic']
    if not type(dst_meta_types) is type([]):
      dst_meta_types = [dst_meta_types]
    self.setDstMetaTypes(dst_meta_types)
    
  def setAttr(self, attr):
    self.attr = attr

  getAttr__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getAttr(self):
    return self.attr

  def setName(self, name):
    self.name = name

  getName__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getName(self):
    return self.name

  def setKeys(self, keys):
    self.keys = keys

  getKeys__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getKeys(self):
    return self.keys

  def setId(self, id):
    self.id = id

  getId__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getId(self):
    return self.id

  def setDstMetaTypes(self, dst_meta_types):
    self.dst_meta_types = dst_meta_types

  getDstMetaTypes__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getDstMetaTypes(self):
    return self.dst_meta_types

  getDatatype__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getDatatype(self):
    if self.getType() == 'multiselect':
      return  'LangList'
    elif self.getType() == 'int':
      return  'LangInt'
    elif self.getType() == 'float':
      return  'LangFloat'
    elif self.getType() == 'date':
      return  'LangDate'
    elif self.getType() == 'time':
      return  'LangTime'
    elif self.getType() == 'datetime':
      return  'LangDateTime'
    elif self.getType() == 'url':
      return  'LangUrl'
    elif self.getType() == 'boolean':
      return  'LangBoolean'
    elif self.getType() == 'image':
      return  'LangImage'
    elif self.getType() == 'file':
      return  'LangFile'
    else:
      return  'LangString'
    
  def setType(self, type):
    self.type = type

  getType__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getType(self):
    return self.type

###################################################################################################
