###################################################################################################
# _metaobjmanager.py
#
# Implementation of class MetaobjManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
import string
import urllib
import copy
# Product Imports.
import _globals
import _xmllib


# -------------------------------------------------------------------------------------------------
#  importMetaObj
# -------------------------------------------------------------------------------------------------

def _importMetaObj(self, item):
  key = item['key']
  value = item['value']
  dtml = item['dtml']
  self.__init_metaobj__(key,value,dtml)
  return key

def importMetaObj(self, xml):
  builder = _xmllib.XmlAttrBuilder()
  value = builder.parse(xml)
  if type(value) is type([]):
    for item in value:
      id = _importMetaObj(self,item)
  else:
    id = _importMetaObj(self,value)
  self.synchronizeObjAttrs()
  return id


# -------------------------------------------------------------------------------------------------
#  findMetaObj
# -------------------------------------------------------------------------------------------------
def findMetaObj(self, meta_id):
  if self.meta_type == 'ZMSCustom' and self.meta_id == meta_id:
    return self
  for child in self.objectValues():
    metaobj = findMetaObj(child,meta_id)
    if metaobj is not None:
      return metaobj
  return None

# -------------------------------------------------------------------------------------------------
#  getTemplateId
# -------------------------------------------------------------------------------------------------
def getTemplateId(id):
  return "bodyContentZMSCustom_%s"%id


###################################################################################################
###################################################################################################
###
###   class MetaobjManager
###
###################################################################################################
###################################################################################################
class MetaobjManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
		'manage_changeMetaobjs','manage_addMetaobj','manage_customizeMetaobjForm',
		)
    __ac_permissions__=(
		('ZMS Administrator', __administratorPermissions__),
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		)


    # Management Interface to customize Meta-Objects.
    manage_customizeMetaobjForm = HTMLFile('dtml/metaobj/manage_customizeform', globals()) 
    metaobj_input_fields = HTMLFile('dtml/metaobj/input_fields', globals())
    metaobj_input_js = HTMLFile('dtml/metaobj/input_js', globals())


    # ------------------------------------------------------------------------------------------
    # MetaobjManager.recurse_updateVersionMetaObjs: 
    #
    # Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersionMetaObjs(self, lang, manage_lang, REQUEST):
      message = ""

      # -----------------------------------------------------------
      # 17.04.2002
      # Build #071: Meta-Object Type.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '071':
        obs = self.__get_metaobjs__()
        for id in obs.keys():
          ob = self.__get_metaobj__(id)
	  tmplt_ids = self.getHome().objectIds(['DTML Method'])
	  if 'ZMSCustom_%s'%id in tmplt_ids:
	    ob['type'] = 'ZMSObject'
	  else:
	    ob['type'] = 'ZMSDocument'
          obs[id] = ob
        self.setConfProperty('ZMS.custom.objects',obs.copy())

      # -----------------------------------------------------------
      # 20.08.2002
      # Build #094: Meta-Object Attribute Keys.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '094':
        obs = self.__get_metaobjs__()
        for id in obs.keys():
          ob = self.__get_metaobj__(id)
          attrs = []
          for attr in ob['__obj_attrs__']:
            attr_keys = ''
            if attr.has_key('keys'):
              attr_keys = attr['keys']
              attr_keys = string.replace(attr_keys,',','\n')
            attr['keys'] = attr_keys
            attrs.append(attr)
          ob['__obj_attrs__'] = attrs
          obs[id] = ob
        self.setConfProperty('ZMS.custom.objects',obs.copy())

      # -----------------------------------------------------------
      # 22.08.2002
      # Build #096: Replace Meta-Object Attribute Type Html by Text.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '096':
        obs = self.__get_metaobjs__()
        for id in obs.keys():
          ob = self.__get_metaobj__(id)
          attrs = []
          for attr in ob['__obj_attrs__']:
            attr_type = attr['type']
            if attr_type == 'LangHtml':
              attr_type = 'LangText'
            attr['type'] = attr_type
            attrs.append(attr)
          ob['__obj_attrs__'] = attrs
          obs[id] = ob
        self.setConfProperty('ZMS.custom.objects',obs.copy())
      
      # -----------------------------------------------------------
      # 16.12.2002
      # Build #108: Move Templates from Home- to Document-Element.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '108':
        obs = self.__get_metaobjs__()
        for obId in obs.keys():
          ob = self.__get_metaobj__(obId)
          obType = ob['type']
          if obType == 'ZMSDocument':
            id = "bodyContentZMSCustom_%s"%obId
          elif obType == 'ZMSTeaserElement':
            id = "bodyContentTeaserZMSCustom_%s"%obId
          else:
            id = "ZMSCustom_%s"%obId
          new_id = getTemplateId(obId)
          homeElmnt = self.getHome()
          docElmnt = self.getDocumentElement()
          cb_copy_data = homeElmnt.manage_cutObjects(ids=[id])
          docElmnt.manage_pasteObjects(cb_copy_data)
          docElmnt.manage_renameObject(id='copy_of_%s'%id,new_id=new_id)

      ##### SYNCHRONIZE ####      
      self.synchronizeObjAttrs()
      
      return message

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__get_metaobjs__:
    # 
    # Returns dictionary of Meta-Objects.
    # ------------------------------------------------------------------------------------------
    def __get_metaobjs__(self):
      return self.getConfProperty('ZMS.custom.objects',{})

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__get_metaobj__:
    # 
    # Returns Meta-Object identified by ID.
    # ------------------------------------------------------------------------------------------
    def __get_metaobj__(self, id): 
      return self.__get_metaobjs__()[id]

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.getMetaobjIds:
    # 
    # Returns list of IDs of Meta-Objects.
    # ------------------------------------------------------------------------------------------
    def getMetaobjIds(self, REQUEST=None):
      obs = self.__get_metaobjs__()
      tmp = []
      for id in obs.keys():
        ob = self.__get_metaobj__(id)
        tmp.append((ob['name'],id))
      tmp.sort()
      ids = []
      for ob in tmp:
        ids.append(ob[1])
      return ids

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.getMetaobj:
    # 
    # Returns Meta-Object identified by ID.
    # ------------------------------------------------------------------------------------------
    def getMetaobj(self, id):
      ob = self.__get_metaobj__(id)
      # Retrieve properties.
      name = ob['name']
      type = ob['type']
      attrs = ob['__obj_attrs__']
      tmplt = getTemplateId(id)
      # Return dictionary of properties.
      return {'id':id,'name':name,'type':type,'attrs':attrs,'tmplt':tmplt}

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.getMetaobjId:
    # 
    # Returns Meta-Object ID identified by Name.
    # ------------------------------------------------------------------------------------------
    def getMetaobjId(self, name):
      for id in self.getMetaobjIds():
        ob = self.getMetaobj(id)
        if name == ob['name']:
          return id
      return None


    """
    ###############################################################################################
    #
    #   A T T R I B U T E S
    #
    ###############################################################################################
    """

    # ------------------------------------------------------------------------------------------
    # MetaobjManager._initMetaObjAttrs:
    # 
    # Init list of Attributes for Meta-Object specified by ID.
    # ------------------------------------------------------------------------------------------
    def _initMetaObjAttrs(self, id):
      attrs = []
      cp = copy.copy(attrs)
      # Init Attributes.
      ob = self.__get_metaobj__(id)
      ob['__obj_attrs__'] = attrs
      # Set Attributes to Meta-Object.
      obs =  self.__get_metaobjs__()
      obs[id] = ob
      self.setConfProperty('ZMS.custom.objects',obs.copy())

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.getMetaobjAttrIds:
    # 
    # Returns list of Attribute-IDs for Meta-Object specified by ID.
    # ------------------------------------------------------------------------------------------
    def getMetaobjAttrIds(self, id):
      ids = []
      ob = self.__get_metaobj__(id)
      attrs = ob['__obj_attrs__']
      for attr in attrs:
        ids.append(attr['id'])
      return ids

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.getMetaobjAttr:
    # 
    # Get Attribute for Meta-Object specified by ID..
    # ------------------------------------------------------------------------------------------
    def getMetaobjAttr(self, id, attr_id):
      ob = self.__get_metaobj__(id)
      attrs = ob['__obj_attrs__']
      for attr in attrs:
        if attr['id'] == attr_id:
          return attr
      return None

    # ------------------------------------------------------------------------------------------
    # MetaobjManager._setMetaObjAttr:
    # 
    # Set/add Meta-Object Attribute with specified values.
    # ------------------------------------------------------------------------------------------
    def _setMetaObjAttr(self, id, attr_id, attr_name, attr_mandatory=0, attr_repetitive=0, attr_type='LangString', attr_keys=''):
      ob = self.__get_metaobj__(id)
      attrs = copy.copy(ob['__obj_attrs__'])
      # Set Attributes.
      if not attr_type in ['LangString','LangText']:
        attr_keys = ''
      if attr_type[:3] == 'ZMS' or attr_type in self.getMetaobjIds():
        attr_mandatory = 0
      else:
        attr_repetitive = 0
      attr = {}
      attr['id'] = attr_id
      attr['name'] = attr_name
      attr['mandatory'] = attr_mandatory
      attr['repetitive'] = attr_repetitive
      attr['type'] = attr_type
      attr['keys'] = attr_keys
      ids = self.getMetaobjAttrIds(id)
      if attr_id in ids:
        attrs[ids.index(attr_id)] = attr
      else:
        attrs.append(attr)
      ob['__obj_attrs__'] = attrs
      # Set Attributes to Meta-Object.
      obs =  self.__get_metaobjs__()
      obs[id] = ob
      self.setConfProperty('ZMS.custom.objects',obs.copy())

    # ------------------------------------------------------------------------------------------
    # MetaobjManager._moveMetaObjAttr:
    # 
    # Moves Meta-Object Attribute in specified direction (-1=up, +1=down).
    # ------------------------------------------------------------------------------------------
    def _moveMetaObjAttr(self, id, attr_id, direction=0):
      ob = self.__get_metaobj__(id)
      attrs = copy.copy(ob['__obj_attrs__'])
      # Move Attributes.
      ids = self.getMetaobjAttrIds(id)
      i = ids.index(attr_id)
      attr = attrs[i]
      attrs[i] = attrs[i+direction]
      attrs[i+direction] = attr
      ob['__obj_attrs__'] = attrs
      # Set Attributes to Meta-Object.
      obs =  self.__get_metaobjs__()
      obs[id] = ob
      self.setConfProperty('ZMS.custom.objects',obs.copy())
      
    # ------------------------------------------------------------------------------------------
    # MetaobjManager._delMetaObjAttr:
    # 
    # Delete Attribute from Meta-Object specified by ID..
    # ------------------------------------------------------------------------------------------
    def _delMetaObjAttr(self, id, attr_id):
      # Delete Attribute.
      ob = self.__get_metaobj__(id)
      cp = []
      attrs = copy.copy(ob['__obj_attrs__'])
      for attr in attrs:
        if attr['id'] != attr_id:
          cp.append(attr)
      ob['__obj_attrs__'] = cp
      # Set Attributes to Meta-Object.
      obs =  self.__get_metaobjs__()
      obs[id] = ob
      self.setConfProperty('ZMS.custom.objects',obs.copy())

    ###############################################################################################

    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__set_metaobj__:
    # 
    # Set/add Meta-Object with specified values.
    # ------------------------------------------------------------------------------------------
    def __init_metaobj__(self, id, ob, dtml_raw=None):
    
      obs =  self.__get_metaobjs__()
      
      # Template.
      # ---------
      docElmnt = self.getDocumentElement()
      dtml_id = getTemplateId(id)
      dtml_title = "Template: %s"%ob['name']
      if dtml_raw is None:
        dtml_raw = ""
        dtml_raw = dtml_raw + "<!-- BO %s -->\n"%dtml_id
        dtml_raw = dtml_raw + "<!-- EO %s -->\n"%dtml_id
      # Insert.
      if not hasattr(docElmnt,dtml_id):
        docElmnt.manage_addDTMLMethod(dtml_id,dtml_title,dtml_raw)
      # Proxy.
      dtml_tmplt = getattr(docElmnt,dtml_id)
      dtml_tmplt.manage_proxy(roles=['Manager'])
      
      # Set Attributes to Meta-Object.
      obs[id] = ob
      self.setConfProperty('ZMS.custom.objects',obs.copy())


    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__set_metaobj__:
    # 
    # Set/add Meta-Object with specified values.
    # ------------------------------------------------------------------------------------------
    def __set_metaobj__(self, id, name, metatype, attrs=[]):
      
      # Init Meta-Object.
      # -----------------
      obs = self.__get_metaobjs__()
      
      # Set Meta-Object.
      # ----------------
      ob = {}
      ob['name'] = name
      ob['type'] = metatype
      ob['__obj_attrs__'] = copy.copy(attrs)
      self.__init_metaobj__(id,ob)


    # ------------------------------------------------------------------------------------------
    # MetaobjManager.__del_metaobj__:
    # 
    # Delete Meta-Object specified by ID.
    # ------------------------------------------------------------------------------------------
    def __del_metaobj__(self, id):
      
      # Delete from attribute values.
      cp = self.__get_metaobjs__()
      obs = {}
      metatype = None
      for cp_id in cp.keys():
        if cp_id == id:
          metatype = cp[cp_id]['type']
        else:
          obs[cp_id] = cp[cp_id]
      
      # Delete Template.
      # ----------------
      docElmnt = self.getDocumentElement()
      dtml_id = getTemplateId(id)
      if hasattr(docElmnt,dtml_id):
        docElmnt.manage_delObjects(ids=[dtml_id])
      
      # Set Attributes to Meta-Object.
      self.setConfProperty('ZMS.custom.objects',obs.copy())


    ###############################################################################################
    # MetaobjManager.manage_changeMetaobjs:
    #
    # Werte fr Meta-Objects lschen oder ndern.
    ###############################################################################################
    def manage_changeMetaobjs(self, lang, manage_lang, btn='', key='all', REQUEST=None, RESPONSE=None):
        """ MetaobjManager.manage_changeMetaobjs """
        
        message = ''
        id = REQUEST.get('id','')
        
        # Delete.
        # -------
        
        # Obj.
        if key == 'obj' and btn == self.getLangStr('BTN_DELETE',manage_lang):
          metaobj = findMetaObj(self,id)
          if metaobj is None:
            self.__del_metaobj__(id)
            id = ''
            message = self.getLangStr('MSG_CHANGED',manage_lang)
          else:
            message = '[ZMS-ERROR]: Constraint violation at <a href="%s/manage_main?lang=%s&manage_lang=%s#_%s">%s</a>'%(metaobj.getParentNode().absolute_url(),lang,manage_lang,metaobj.id,metaobj.absolute_url())
        
        # Attr.
        if key == 'attr' and btn == 'delete':
          attr_id = REQUEST['attr_id']
          self._delMetaObjAttr(id,attr_id)
        
        # Change.
        # -------
        elif key == 'all' and btn == self.getLangStr('BTN_CHANGE',manage_lang):
        
          # Obj.
          id = REQUEST['id']
          name = REQUEST['obj_name']
          metatype = REQUEST['obj_type']
          attrs = ob = self.__get_metaobj__(id)
          attrs = ob['__obj_attrs__']
          self.__set_metaobj__(id,name,metatype,attrs)
          meta_obj = self.getMetaobj(id)
          if hasattr(self,meta_obj['tmplt']):
            ob = getattr(self,meta_obj['tmplt'])
            ob.manage_edit(title='Template: %s'%meta_obj['name'],data=REQUEST.get('obj_dtml',''))
            ob.manage_proxy(roles=['Manager'])
          
          # Attr.
          saved_ids = self.getMetaobjAttrIds(id)
          self._initMetaObjAttrs(id)
          for saved_id in saved_ids:
            attr_id = REQUEST['attr_id_%s'%saved_id]
            attr_name = REQUEST['attr_name_%s'%saved_id]
            attr_mandatory = REQUEST.get('attr_mandatory_%s'%saved_id,0)
            attr_repetitive = REQUEST.get('attr_repetitive_%s'%saved_id,0)
            attr_type = REQUEST['attr_type_%s'%saved_id]
            attr_keys = REQUEST.get('attr_keys_%s'%saved_id,'')
            self._setMetaObjAttr(id,attr_id,attr_name,attr_mandatory,attr_repetitive,attr_type,attr_keys)
          
          message = self.getLangStr('MSG_CHANGED',manage_lang)
        
        # Export.
        # -------
        elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
          value = []
          ids = REQUEST.get('ids',[])
          for id in self.__get_metaobjs__().keys():
            if id in ids or len(ids) == 0:
              ob = self.__get_metaobj__(id)
              docElmnt = self.getDocumentElement()
              dtml_id = getTemplateId(id)
              dtml_raw = ""
              for method in docElmnt.objectValues(['DTML Method']):
                if method.id() == dtml_id:
                  dtml_raw = method.raw
              value.append({'key':id,'value':ob,'dtml':dtml_raw})
          if len(value)==1:
            value = value[0]
          content_type = 'text/xml'
          filename = id + '.xml'
          export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
          export = export + _xmllib.toXml(self,value,1)
          RESPONSE.setHeader('Content-Type',content_type)
          RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
          return export
        
        # Insert.
        # -------
        elif btn == self.getLangStr('BTN_INSERT',manage_lang):
        
          # Obj.
          if key == 'obj':
            id = string.strip(REQUEST['_meta_id'])
            name = string.strip(REQUEST['_meta_name'])
            metatype = string.strip(REQUEST['_meta_type'])
            self.__set_metaobj__(id,name,metatype)
            if metatype == 'ZMSDocument':
              self._setMetaObjAttr(id,'titlealt',self.getLangStr('ATTR_TITLESHORT',manage_lang),1,0,'LangString')
              self._setMetaObjAttr(id,'title',self.getLangStr('ATTR_TITLE',manage_lang),1,0,'LangString')
            elif metatype == 'ZMSTeaserElement':
              self._setMetaObjAttr(id,'titlealt',self.getLangStr('ATTR_TITLESHORT',manage_lang),1,0,'LangString')
              self._setMetaObjAttr(id,'attr_penetrance',self.getLangStr('ATTR_PENETRANCE',manage_lang),1,0,'LangString','this\nsub_nav\nsub_all')
          
          # Attr.
          if key == 'attr':
            attr_id = REQUEST['attr_id']
            attr_name = REQUEST['attr_name']
            attr_type = REQUEST['attr_type']
            attr_mandatory = REQUEST.get('_mandatory',0)
            attr_repetitive = REQUEST.get('_repetitive',0)
            attr_keys = REQUEST.get('_keys','')
            self._setMetaObjAttr(id,attr_id,attr_name,attr_mandatory,attr_repetitive,attr_type,attr_keys)
        
        # Import.
        # -------
        elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
          id = importMetaObj(self,xml=REQUEST['_meta_file'])
          message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%REQUEST['_meta_file'].filename)

        # Move up.
        # --------
        elif key == 'attr' and btn == 'move_up':
          attr_id = REQUEST['attr_id']
          self._moveMetaObjAttr(id,attr_id,-1)
          message = self.getLangStr('MSG_MOVEDOBJUP',manage_lang)%("<i>%s</i>"%attr_id)
        
        # Move down.
        # ----------
        elif key == 'attr' and btn == 'move_down':
          attr_id = REQUEST['attr_id']
          self._moveMetaObjAttr(id,attr_id,+1)
          message = self.getLangStr('MSG_MOVEDOBJDOWN',manage_lang)%("<i>%s</i>"%attr_id)
        
        ##### SYNCHRONIZE ####      
        self.synchronizeObjAttrs()
        
        # Return with message.
        message = urllib.quote(message)
        return RESPONSE.redirect('manage_customizeMetaobjForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s&attr_id=%s'%(lang,manage_lang,message,id,REQUEST.get('attr_id','')))

###################################################################################################
