###################################################################################################
# _zmsglobals.py
#
# Implementation of class ZMSGlobals (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from App.Common import package_home
from Globals import HTMLFile
from DateTime.DateTime import DateTime
import DocumentTemplate.DT_HTML
import urllib
import time
import string 
import sys
# Product Imports.
import _fileutil
import _globals


###################################################################################################
###################################################################################################
###
###   class ZMSGlobals:
###
###################################################################################################
###################################################################################################
class ZMSGlobals: 

    """
    ###############################################################################################
    ###  
    ###  G l o b a l   D e f i n i t i o n s
    ### 
    ###############################################################################################
    """    
    
    # ---------------------------------------------------------------------------------------------
    # Meta-Type Selectors.
    # ---------------------------------------------------------------------------------------------
    TYPE_METATYPE_SELECTOR = type(0)
    PAGES = 0
    PAGEELEMENTS = 1
    RESOURCE = 2
    NORESOURCE = 3
    NOREF = 4
    NORESOLVEREF = 5


    """
    ###############################################################################################
    ###  
    ###  G l o b a l   F u n c t i o n s
    ### 
    ###############################################################################################
    """    
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.rand_int:
    # ---------------------------------------------------------------------------------------------
    def rand_int(self, n):
      from whrandom import randint
      return randint(0,n)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.string_maxlen:
    # ---------------------------------------------------------------------------------------------
    def string_maxlen(self, s, maxlen=20):
      if len(s) > maxlen:
        s = s[:maxlen] + "&middot;&middot;&middot;"
      return s

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.url_quote:
    # ---------------------------------------------------------------------------------------------
    def url_quote(self, s):
      return urllib.quote(s)
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.url_inherit_params:
    # ---------------------------------------------------------------------------------------------
    def url_inherit_params(self, s, REQUEST, exclude=[]):
      rtn = s
      if REQUEST.form:
        count = 0
        for key in REQUEST.form.keys():
          if count == 0:
            rtn = rtn + "?"
          else:
            rtn = rtn + "&"
          rtn = rtn + key + "=" + urllib.quote(REQUEST.form[key])
          count = count + 1    	  
      return rtn

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.string_list:
    # ---------------------------------------------------------------------------------------------
    def string_list(self, s, sep='\n'):
      l = []
      for i in string.split(s,'\n'):
        while len(i) > 0 and ord(i[-1]) < 32:
          i = i[:-1]
        if len(i) > 0:
          l.append(i)
      return l

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.id_quote:
    # ---------------------------------------------------------------------------------------------
    def id_quote(self, s):
      return _globals.id_quote(s)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.js_quote:
    # ---------------------------------------------------------------------------------------------
    def js_quote(self, s):
      return _globals.js_quote(s)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.isPreviewRequest:
    # ---------------------------------------------------------------------------------------------
    def isPreviewRequest(self, REQUEST):
      return _globals.isPreviewRequest(REQUEST)



    """
    ###############################################################################################
    ###  
    ###  H e l p   C o n t e x t
    ### 
    ###############################################################################################
    """    
    
    # Management Interface.
    # ---------------------
    f_help = HTMLFile('dtml/object/f_help', globals()) 



    """
    ###############################################################################################
    #
    #  D A T E   T I M E
    #
    ###############################################################################################
    """

    # ==========================================================================
    # Index  Field  Values  
    # 0  year (for example, 1993) 
    # 1  month range [1,12] 
    # 2  day range [1,31] 
    # 3  hour range [0,23] 
    # 4  minute range [0,59] 
    # 5  second range [0,61]; see (1) in strftime() description 
    # 6  weekday range [0,6], Monday is 0 
    # 7  Julian day range [1,366] 
    # 8  daylight savings flag 0, 1 or -1; see below 
    # ==========================================================================

    # -------------------------------------------------------------------------------------------------
    #	Language.getLangFmtDate:
    # -------------------------------------------------------------------------------------------------
    def getLangFmtDate(self, t, lang, fmt_str='DATETIME_FMT'):
      try:
        t = _globals.getDateTime(t)
        if fmt_str == 'Day':
          return self.getLangStr('WEEKDAY%i'%t[6],lang)
        elif fmt_str == 'Month':
          return self.getLangStr('MONTH%i'%t[1],lang)
        else:
          fmt = self.getLangStr(fmt_str,lang)
          time_fmt = self.getLangStr('TIME_FMT',lang)
          date_fmt = self.getLangStr('DATE_FMT',lang)
          HOURS = 3
          MINUTES = 4
          SECONDS = 5
          if string.find(fmt,time_fmt) >= 0:
            if t[HOURS] == 0 and \
               t[MINUTES] == 0 and \
               t[SECONDS]== 0:
              fmt = fmt[:-len(time_fmt)]
            elif t[SECONDS] == 0:
              fmt = fmt[:-3]
          fmt = string.strip(fmt)
          return time.strftime(fmt,t)
      except:
        return str(t)


    # -------------------------------------------------------------------------------------------------
    #	Language.parseLangFmtDate:
    # -------------------------------------------------------------------------------------------------
    def parseLangFmtDate(self, s, lang, fmt_str='DATETIME_FMT', recflag=0):
      value = None
      if s is not None and len(str(s)) > 0:
        sTime = str(s)
        dctTime = {'Y':0,'m':0,'d':0,'H':0,'M':0,'S':0}
        fmt = self.getLangStr(fmt_str,lang)
        try:
          while string.find(fmt,'%') >=0 and len(sTime) > 0:
            i = string.find(fmt,'%')+1
            sDirective = fmt[i]
            fmt = fmt[i+1:]
            if len(fmt) == 0 or string.find(sTime,fmt[0]) < 0:
              if string.find(sTime,'.') > 0:
                sTime = sTime[:string.find(sTime,'.')]
              elif string.find(sTime,'GMT') > 0:
                sTime = sTime[:string.find(sTime,'GMT')]
              dctTime[sDirective] = int(string.strip(sTime))
              sTime = ''
            else:
              i = string.find(sTime,fmt[0])
              dctTime[sDirective] = int(string.strip(sTime[:i]))
              sTime = sTime[i+1:]
          value = (dctTime['Y'],dctTime['m'],dctTime['d'],dctTime['H'],dctTime['M'],dctTime['S'],0,1,-1)
        except:
          if recflag:
            if _globals.debug:
              print "[parseLangFmtDate]: sTime=%s; fmt=%s"%(sTime,fmt)
              print "Type: ",sys.exc_type
              print "Value: ",sys.exc_value
              print "Traceback: ",sys.exc_traceback
        if value is None and not recflag:
          langAlt = 'ger'
          if lang == langAlt:
            langAlt = 'eng'
          value = self.parseLangFmtDate(s,langAlt,fmt_str,1)
        value = _globals.getDateTime(value)
      return value

###################################################################################################
